@@ -20,23 +20,29 @@
 #define GPARTED_FILESYSTEM_H
 
 #include "../include/Operation.h"
+#include "../include/Partition.h"
 #include "../include/PipeCapture.h"
 
 #include <fstream>
 #include <sys/stat.h>
+#include <sigc++/slot.h>
 
 namespace GParted
 {
 
 enum ExecFlags
 {
-	EXEC_NONE         = 1 << 0,
-	EXEC_CHECK_STATUS = 1 << 1,  // Set the status of the command in the operation
-	                             // details based on the exit status being zero or
-	                             // non-zero.  Must either use this flag when calling
-	                             // ::execute_command() or call ::set_status()
-	                             // afterwards.
-	EXEC_CANCEL_SAFE  = 1 << 2
+	EXEC_NONE            = 1 << 0,
+	EXEC_CHECK_STATUS    = 1 << 1,  // Set the status of the command in the operation
+	                                // details based on the exit status being zero or
+	                                // non-zero.  Must either use this flag when calling
+	                                // ::execute_command() or call ::set_status()
+	                                // afterwards.
+	EXEC_CANCEL_SAFE     = 1 << 2,
+	EXEC_PROGRESS_STDOUT = 1 << 3,  // Run progress tracking callback after reading new
+	                                // data on stdout from command.
+	EXEC_PROGRESS_STDERR = 1 << 4,  // Same but for stderr.
+	EXEC_PROGRESS_TIMED  = 1 << 5   // Run progress tracking callback periodically.
 };
 
 inline ExecFlags operator|( ExecFlags lhs, ExecFlags rhs )
@@ -76,8 +82,17 @@ public:
 	virtual bool remove( const Partition & partition, OperationDetail & operationdetail ) { return true; };
 
 protected:
+	typedef sigc::slot<void, OperationDetail *> StreamSlot;
+	typedef sigc::slot<bool, OperationDetail *> TimedSlot;
+
 	int execute_command( const Glib::ustring & command, OperationDetail & operationdetail,
 	                     ExecFlags flags = EXEC_NONE );
+	int execute_command( const Glib::ustring & command, OperationDetail & operationdetail,
+	                     ExecFlags flags,
+	                     StreamSlot stream_progress_slot );
+	int execute_command( const Glib::ustring & command, OperationDetail & operationdetail,
+	                     ExecFlags flags,
+	                     TimedSlot timed_progress_slot );
 	void set_status( OperationDetail & operationdetail, bool success );
 	void execute_command_eof();
 	Glib::ustring mk_temp_dir( const Glib::ustring & infix, OperationDetail & operationdetail ) ;
@@ -87,11 +102,12 @@ protected:
 	Glib::ustring output, error ;
 	Sector T, N, S ;  //File system [T]otal num of blocks, [N]um of free (or used) blocks, block [S]ize
 	int exit_status ;
-	unsigned int index ;
-	sigc::signal<void, OperationDetail *> signal_progress;
 
 private:
-	void update_command_progress( OperationDetail *operationdetail );
+	int execute_command_internal( const Glib::ustring & command, OperationDetail & operationdetail,
+	                              ExecFlags flags,
+	                              StreamSlot stream_progress_slot,
+	                              TimedSlot timed_progress_slot );
 	void store_exit_status( GPid pid, int status );
 	bool running;
 	int pipecount;
