@@ -18,8 +18,12 @@
 #ifndef GPARTED_GPARTED_CORE_H
 #define GPARTED_GPARTED_CORE_H
 
+#include "../include/BlockSpecial.h"
 #include "../include/FileSystem.h"
 #include "../include/Operation.h"
+#include "../include/Partition.h"
+#include "../include/PartitionLUKS.h"
+#include "../include/PartitionVector.h"
 
 #include <parted/parted.h>
 #include <vector>
@@ -56,8 +60,6 @@ public:
 	const std::vector<FS> & get_filesystems() const ;
 	const FS & get_fs( GParted::FILESYSTEM filesystem ) const ;
 	static std::vector<Glib::ustring> get_disklabeltypes() ;
-	static bool is_dev_mounted( const Glib::ustring & path ) ;
-	static std::vector<Glib::ustring> get_all_mountpoints() ;
 	std::map<Glib::ustring, bool> get_available_flags( const Partition & partition ) ;
 	Glib::ustring get_libparted_version() ;
 	Glib::ustring get_thread_status_message() ;
@@ -65,40 +67,32 @@ public:
 	static FileSystem * get_filesystem_object( FILESYSTEM filesystem );
 	static bool supported_filesystem( FILESYSTEM fstype );
 	static bool filesystem_resize_disallowed( const Partition & partition ) ;
+	static void insert_unallocated( const Glib::ustring & device_path,
+	                                PartitionVector & partitions,
+	                                Sector start,
+	                                Sector end,
+	                                Byte_Value sector_size,
+	                                bool inside_extended );
+
 private:
 	//detectionstuff..
-	static void init_maps() ;
 	void set_thread_status_message( Glib::ustring msg ) ;
-	static void read_mountpoints_from_file( const Glib::ustring & filename,
-	                                        std::map< Glib::ustring, std::vector<Glib::ustring> > & map ) ;
-	static void add_node_and_mountpoint( std::map< Glib::ustring, std::vector<Glib::ustring> > & map,
-	                                     Glib::ustring & node,
-	                                     Glib::ustring & mountpoint ) ;
-	static void read_mountpoints_from_file_swaps( const Glib::ustring & filename,
-	                                              std::map< Glib::ustring, std::vector<Glib::ustring> > & map ) ;
-	static bool have_rootfs_dev( std::map< Glib::ustring, std::vector<Glib::ustring> > & map ) ;
-	static void read_mountpoints_from_mount_command( std::map< Glib::ustring, std::vector<Glib::ustring> > & map ) ;
 	static Glib::ustring get_partition_path( PedPartition * lp_partition );
 	void set_device_serial_number( Device & device );
 	void set_device_partitions( Device & device, PedDevice* lp_device, PedDisk* lp_disk ) ;
 	void set_device_one_partition( Device & device, PedDevice * lp_device, FILESYSTEM fstype,
 	                               std::vector<Glib::ustring> & messages );
+	void set_luks_partition( PartitionLUKS & partition );
 	void set_partition_label_and_uuid( Partition & partition );
 	static FILESYSTEM detect_filesystem_internal( PedDevice * lp_device, PedPartition * lp_partition );
 	static FILESYSTEM detect_filesystem( PedDevice * lp_device, PedPartition * lp_partition,
 	                                     std::vector<Glib::ustring> & messages );
 	void read_label( Partition & partition ) ;
 	void read_uuid( Partition & partition ) ;
-	void insert_unallocated( const Glib::ustring & device_path,
-				 std::vector<Partition> & partitions,
-				 Sector start,
-				 Sector end,
-				 Byte_Value sector_size,
-				 bool inside_extended ) ;
-	void set_mountpoints( std::vector<Partition> & partitions ) ;
-	bool set_mountpoints_helper( Partition & partitions, const Glib::ustring & path ) ;
+	void set_mountpoints( Partition & partition );
+	bool set_mountpoints_helper( Partition & partition, const Glib::ustring & path );
 	bool is_busy( FILESYSTEM fstype, const Glib::ustring & path ) ;
-	void set_used_sectors( std::vector<Partition> & partitions, PedDisk* lp_disk ) ;
+	void set_used_sectors( Partition & partition, PedDisk* lp_disk );
 	void mounted_set_used_sectors( Partition & partition ) ;
 #ifdef HAVE_LIBPARTED_FS_RESIZE
 	void LP_set_used_sectors( Partition & partition, PedDisk* lp_disk ) ;
