@@ -24,6 +24,9 @@
 #define GPARTED_PARTITION_H
 
 #include "../include/Utils.h"
+#include "../include/PartitionVector.h"
+
+#include <glibmm/ustring.h>
 
 namespace GParted
 {
@@ -50,12 +53,20 @@ enum PartitionAlignment {
 	                       //  Indicator if start and end sectors must remain unchanged
 };
 
+class Partition;        // Forward declarations as Partition and PartitionVector are
+class PartitionVector;  // mutually recursive classes.
+                        // References:
+                        // *   Mutually recursive classes
+                        //     http://stackoverflow.com/questions/3410637/mutually-recursive-classes
+                        // *   recursive definition in CPP
+                        //     http://stackoverflow.com/questions/4300420/recursive-definition-in-cpp
+
 class Partition
 {
 public:
 	Partition() ;
-	Partition( const Glib::ustring & path ) ;
-	~Partition() ;
+	virtual ~Partition();
+	virtual Partition * clone() const;
 
 	void Reset() ;
 	
@@ -73,11 +84,11 @@ public:
 	          bool busy );
 
 	void set_sector_usage( Sector sectors_fs_size, Sector sectors_fs_unused ) ;
-	bool sector_usage_known() const ;
-	Sector estimated_min_size() const ;
-	Sector get_sectors_used() const ;
-	Sector get_sectors_unused() const ;
-	Sector get_sectors_unallocated() const ;
+	virtual bool sector_usage_known() const;
+	virtual Sector estimated_min_size() const;
+	virtual Sector get_sectors_used() const;
+	virtual Sector get_sectors_unused() const;
+	virtual Sector get_sectors_unallocated() const;
 	void get_usage_triple( int imax, int & i1, int & i2, int & i3 ) const ;
 
 	void Set_Unallocated( const Glib::ustring & device_path,
@@ -90,12 +101,10 @@ public:
 	//update partition number (used when a logical partition is deleted) 
 	void Update_Number( int new_number );
 	
-	void add_path( const Glib::ustring & path, bool clear_paths = false ) ;
-	void add_paths( const std::vector<Glib::ustring> & paths, bool clear_paths = false ) ;
+	void set_path( const Glib::ustring & path );
 	Byte_Value get_byte_length() const ;
 	Sector get_sector_length() const ; 
 	Glib::ustring get_path() const ;
-	std::vector<Glib::ustring> get_paths() const ;
 	void add_mountpoint( const Glib::ustring & mountpoint, bool clear_mountpoints = false ) ;
 	void add_mountpoints( const std::vector<Glib::ustring> & mountpoints, bool clear_mountpoints = false ) ;
 	Glib::ustring get_mountpoint() const ; 
@@ -104,9 +113,24 @@ public:
 	Sector get_sector() const ;
 	bool test_overlap( const Partition & partition ) const ;
 	bool filesystem_label_known() const;
-	Glib::ustring get_filesystem_label() const;
+	virtual Glib::ustring get_filesystem_label() const;
 	void set_filesystem_label( const Glib::ustring & filesystem_label );
 
+	// Message accessors.  Messages are stored locally and accessed globally.
+	// Stored locally means the messages are stored in the Partition object to which
+	// they are added so push_back_messages() and append_messages() are non-virtual.
+	// Accessed globally means that in the case of derived objects which are composed
+	// of more that one Partition object, i.e. PartitionLUKS, the messages have to be
+	// accessible from all copies within the derived object hierarchy.  Hence
+	// have_messages(), get_messages() and clear_messages() are virtual to allow for
+	// overridden implementations.
+	virtual bool have_messages() const                             { return ! messages.empty(); };
+	virtual std::vector<Glib::ustring> get_messages() const        { return messages; };
+	virtual void clear_messages()                                  { messages.clear(); };
+	void push_back_message( Glib::ustring msg )                    { messages.push_back( msg ); };
+	void append_messages( const std::vector<Glib::ustring> msgs )
+	                                { messages.insert( messages.end(), msgs.begin(), msgs.end() ); }
+
 	bool operator==( const Partition & partition ) const ;
 	bool operator!=( const Partition & partition ) const ;
 		
@@ -126,31 +150,30 @@ public:
 	Sector sectors_unused;
 	Sector sectors_unallocated;  //Difference between the size of the partition and the file system
 	Sector significant_threshold;  //Threshold from intrinsic to significant unallocated sectors
-	Gdk::Color color;
 	bool inside_extended;
 	bool busy;
-	std::vector<Glib::ustring> messages ;
 	std::vector<Glib::ustring> flags ;
-	
-	std::vector<Partition> logicals ;
+
+	PartitionVector logicals;
 
 	bool strict_start ;	//Indicator if start sector must stay unchanged
 	Sector free_space_before ;  //Free space preceding partition value
 
 	Byte_Value sector_size ;  //Sector size of the disk device needed for converting to/from sectors and bytes.
+	Byte_Value fs_block_size;  // Block size of of the file system, or -1 when unknown.
 
 private:
+	Partition & operator=( Partition & rhs );  // Not implemented copy assignment operator
+
 	static void get_usage_triple_helper( Sector stot, Sector s1, Sector s2, Sector s3, int imax, int & i1, int & i2, int & i3 ) ;
 
-	void sort_paths_and_remove_duplicates() ;
 	Sector calc_significant_unallocated_sectors() const ;
 
-	static bool compare_paths( const Glib::ustring & A, const Glib::ustring & B ) ;
-	
-	std::vector<Glib::ustring> paths ;
+	Glib::ustring path;
 	std::vector<Glib::ustring> mountpoints ;
 	bool have_filesystem_label;
 	Glib::ustring filesystem_label;
+	std::vector<Glib::ustring> messages;
 };
 
 }//GParted
