@@ -18,6 +18,7 @@
 #define GPARTED_TREEVIEW_DETAIL_H
 
 #include "../include/Partition.h"
+#include "../include/PartitionVector.h"
 
 #include <gtkmm/treeview.h>
 #include <gtkmm/treestore.h>
@@ -35,7 +36,7 @@ class TreeView_Detail : public Gtk::TreeView
 {
 public:
 	TreeView_Detail();
-	void load_partitions( const std::vector<Partition> & partitions ) ;
+	void load_partitions( const PartitionVector & partitions );
 	void set_selected( const Partition * partition_ptr );
 	void clear() ;
 
@@ -45,7 +46,7 @@ public:
 	sigc::signal< void, unsigned int, unsigned int > signal_popup_menu ;
 
 private:
-	void load_partitions( const std::vector<Partition> & partitions,
+	void load_partitions( const PartitionVector & partitions,
 	                      bool & mountpoints,
 	                      bool & labels,
 	                      bool & names,
@@ -76,8 +77,6 @@ private:
 		Gtk::TreeModelColumn<Glib::ustring> used;
 		Gtk::TreeModelColumn<Glib::ustring> unused;
 		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > color ;
-		Gtk::TreeModelColumn<Glib::ustring> text_color;
-		Gtk::TreeModelColumn<Glib::ustring> mount_text_color;
 		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > icon1 ;
 		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > icon2 ;
 		Gtk::TreeModelColumn<Glib::ustring> flags;
@@ -88,8 +87,7 @@ private:
 		{
 			add( path ); add( name ); add( filesystem ); add( mountpoint ); add( label );
 			add( size ); add( used ); add( unused ); add( color );
-			add( text_color ); add( mount_text_color ); add( icon1 );
-			add( icon2 ); add( flags ); add( partition_ptr );
+			add( icon1 ); add( icon2 ); add( flags ); add( partition_ptr );
 		}
 	};
 	
