@@ -55,6 +55,8 @@ const Byte_Value EXBIBYTE=(PEBIBYTE * KIBIBYTE);
 const Glib::ustring UUID_RANDOM = _("(New UUID - will be randomly generated)") ;
 const Glib::ustring UUID_RANDOM_NTFS_HALF = _("(Half new UUID - will be randomly generated)") ;
 
+extern const Glib::ustring DEV_MAPPER_PATH;
+
 enum FILESYSTEM
 {
 	// Special partition types and functions
@@ -77,25 +79,26 @@ enum FILESYSTEM
 	FS_HFSPLUS         = 14,
 	FS_JFS             = 15,
 	FS_LINUX_SWAP      = 16,
-	FS_LVM2_PV         = 17,
-	FS_NILFS2          = 18,
-	FS_NTFS            = 19,
-	FS_REISER4         = 20,
-	FS_REISERFS        = 21,
-	FS_UFS             = 22,
-	FS_XFS             = 23,
+	FS_LUKS            = 17,
+	FS_LVM2_PV         = 18,
+	FS_NILFS2          = 19,
+	FS_NTFS            = 20,
+	FS_REISER4         = 21,
+	FS_REISERFS        = 22,
+	FS_UFS             = 23,
+	FS_XFS             = 24,
 
 	// Recognised signatures but otherwise unsupported file system types
-	FS_BITLOCKER       = 24,
-	FS_LUKS            = 25,
-	FS_LINUX_SWRAID    = 26,
-	FS_LINUX_SWSUSPEND = 27,
-	FS_REFS            = 28,
-	FS_ZFS             = 29,
+	FS_BITLOCKER       = 25,
+	FS_GRUB2_CORE_IMG  = 26,
+	FS_LINUX_SWRAID    = 27,
+	FS_LINUX_SWSUSPEND = 28,
+	FS_REFS            = 29,
+	FS_ZFS             = 30,
 
 	// Partition space usage colours
-	FS_USED            = 30,
-	FS_UNUSED          = 31
+	FS_USED            = 31,
+	FS_UNUSED          = 32
 } ;
 
 enum SIZE_UNIT
@@ -151,10 +154,10 @@ struct FS
 	Byte_Value MIN ; 
 	Byte_Value MAX ;
 	
-	FS()
+	FS( FILESYSTEM fstype = FS_UNKNOWN ) : filesystem( fstype )
 	{
-		read = read_label = write_label = read_uuid = write_uuid = create = create_with_label =
-		grow = shrink = move = check = copy = remove = online_read =
+		busy = read = read_label = write_label = read_uuid = write_uuid = create =
+		create_with_label = grow = shrink = move = check = copy = remove = online_read =
 		online_grow = online_shrink = NONE ;
 		MIN = MAX = 0 ;
 	} 
@@ -195,6 +198,7 @@ public:
 	                                 , const Glib::ustring & pattern
 	                                 ) ;
 	static Glib::ustring trim( const Glib::ustring & src, const Glib::ustring & c = " \t\r\n" ) ;
+	static Glib::ustring last_line( const Glib::ustring & src );
 	static Glib::ustring get_lang() ;
 	static void tokenize( const Glib::ustring& str,
 	                      std::vector<Glib::ustring>& tokens,
