@@ -21,6 +21,7 @@
 #include "../include/Device.h"
 #include "../include/DrawingAreaVisualDisk.h"
 #include "../include/Partition.h"
+#include "../include/PartitionVector.h"
 #include "../include/TreeView_Detail.h"
 #include "../include/GParted_Core.h"
 #include "../include/HBoxOperations.h" 
@@ -49,8 +50,12 @@ class Win_GParted : public Gtk::Window
 {
 public:
 	Win_GParted( const std::vector<Glib::ustring> & user_devices ) ;
+	~Win_GParted();
 
 private:
+	Win_GParted( const Win_GParted & src );              // Not implemented copy constructor
+	Win_GParted & operator=( const Win_GParted & rhs );  // Not implemented copy assignment operator
+
 	void init_menubar() ;
 	void init_toolbar() ;
 	void init_partition_menu() ;
@@ -148,7 +153,8 @@ private:
 	void radio_devices_changed( unsigned int item ) ;
 	bool on_delete_event( GdkEventAny* ) ;
 	void on_show() ;
-		
+
+	static gboolean initial_device_refresh( gpointer data );
 	void menu_gparted_refresh_devices();
 	void menu_gparted_features();
 	void menu_gparted_quit();
@@ -191,11 +197,11 @@ private:
 
 //private variables
 	unsigned int current_device ;
-	std::vector<Partition> display_partitions;  // Copy of current device's partitions with any pending
+	PartitionVector display_partitions;         // Copy of current device's partitions with any pending
 	                                            // operations applied, as currently being shown in the GUI.
 	const Partition * selected_partition_ptr;   // Pointer to the selected partition.  (Alias to element
 	                                            // in Win_GParted::display_partitions[] vector).
-	Partition copied_partition;
+	const Partition * copied_partition;         // NULL or copy of source partition object.
 	std::vector<Device> devices;
 	std::vector<Operation *> operations;
 
