@@ -20,6 +20,7 @@
 #define GPARTED_HFSPLUS_H
 
 #include "../include/FileSystem.h"
+#include "../include/Partition.h"
 
 namespace GParted
 {
@@ -28,7 +29,6 @@ class hfsplus : public FileSystem
 {
 public:
 	FS get_filesystem_support() ;
-	void read_label( Partition & partition ) ;
 	bool create( const Partition & new_partition, OperationDetail & operationdetail ) ;
 	bool check_repair( const Partition & partition, OperationDetail & operationdetail ) ;
 };
