@@ -17,6 +17,9 @@
  
 #include "../include/Dialog_Partition_Info.h"
 #include "../include/LVM2_PV_Info.h"
+#include "../include/Partition.h"
+#include "../include/PartitionLUKS.h"
+#include "../include/Utils.h"
 #include "../include/btrfs.h"
 
 #include <gtk/gtk.h>
@@ -27,13 +30,19 @@ namespace GParted
 
 Dialog_Partition_Info::Dialog_Partition_Info( const Partition & partition ) : partition( partition )
 {
+	// Filesystem points to the Partition object, or for an open LUKS mapping, the
+	// encrypted Partition object member within containing the encrypted file system.
+	filesystem = &partition;
+	if ( partition.filesystem == FS_LUKS && partition.busy )
+		filesystem = &dynamic_cast<const PartitionLUKS *>( &partition )->get_encrypted();
+
 	this ->set_has_separator( false ) ;
-	//Set minimum dialog height so it fits on an 800x600 screen without
-	//  too much whitespace (~500 px max for GNOME desktop).
-	//  Allow extra space if have messages or LVM2 PV
-	if (   partition .messages .size() > 0
-	    || partition .filesystem == FS_LVM2_PV
-	   )
+	// Set minimum dialog height so it fits on an 800x600 screen without too much
+	// whitespace (~500 px max for GNOME desktop).  Allow extra space if have any
+	// messages or for LVM2 PV or LUKS encryption.
+	if ( partition.have_messages()          ||
+	     partition.filesystem == FS_LVM2_PV ||
+	     partition.filesystem == FS_LUKS       )
 		this ->set_size_request( -1, 460) ;
 	else
 		this ->set_size_request( -1, 370 ) ;	//Minimum 370 to avoid scrolling on Fedora 20
@@ -64,7 +73,7 @@ Dialog_Partition_Info::Dialog_Partition_Info( const Partition & partition ) : pa
 	Display_Info() ;
 	
 	//display messages (if any)
-	if ( partition .messages .size() > 0 )
+	if ( partition.have_messages() )
 	{
 		frame = manage( new Gtk::Frame() );
 
@@ -81,29 +90,30 @@ Dialog_Partition_Info::Dialog_Partition_Info( const Partition & partition ) : pa
 
 		frame ->set_label_widget( *hbox ) ;
 
-		//Merge all messages for display so that they can be selected together.
-		//  Use a blank line between individual messages so that each message can be
-		//  distinguished. Therefore each message should have been formatted as one
-		//  or more non-blank lines, with an optional trailing new line.  This is
-		//  true of GParted internal messages and probably all external messages and
-		//  errors from libparted and executed commands too.
-		Glib::ustring all_messages ;
-		for ( unsigned int t = 0; t < partition .messages .size(); t ++ )
+		// Concatenate all messages for display so that they can be selected
+		// together.  Use a blank line between individual messages so that each
+		// message can be distinguished. Therefore each message should have been
+		// formatted as one or more non-blank lines, with an optional trailing new
+		// line.  This is true of GParted internal messages and probably all
+		// external messages and errors from libparted and executed commands too.
+		std::vector<Glib::ustring> messages = partition.get_messages();
+		Glib::ustring concatenated_messages;
+		for ( unsigned int i = 0; i < messages.size(); i ++ )
 		{
-			if ( all_messages .size() > 0 )
-				all_messages += "\n\n" ;
+			if ( concatenated_messages.size() > 0 )
+				concatenated_messages += "\n\n";
 
-			Glib::ustring::size_type take = partition .messages[ t ] .size() ;
+			Glib::ustring::size_type take = messages[i].size();
 			if ( take > 0 )
 			{
-				if ( partition .messages[ t ][ take-1 ] == '\n' )
+				if ( messages[i][take-1] == '\n' )
 					take -- ;  //Skip optional trailing new line
-				all_messages += "<i>" + partition .messages[ t ] .substr( 0, take ) + "</i>" ;
+				concatenated_messages += "<i>" + messages[i].substr( 0, take ) + "</i>";
 			}
 		}
 		Gtk::VBox *vbox( manage( new Gtk::VBox() ) ) ;
 		vbox ->set_border_width( 5 ) ;
-		vbox ->pack_start( *Utils::mk_label( all_messages, true, true, true ), Gtk::PACK_SHRINK ) ;
+		vbox->pack_start( *Utils::mk_label( concatenated_messages, true, true, true ), Gtk::PACK_SHRINK );
 		frame ->add( *vbox ) ;
 
 		info_msg_vbox .pack_start( *frame, Gtk::PACK_EXPAND_WIDGET ) ;
@@ -192,8 +202,8 @@ void Dialog_Partition_Info::init_drawingarea()
 
 	color_text .set( "black" );
 	this ->get_colormap() ->alloc_color( color_text ) ;
-	
-	color_partition = partition .color ;
+
+	color_partition.set( Utils::get_color( filesystem->filesystem ) );
 	this ->get_colormap() ->alloc_color( color_partition ) ;	 
 	
 	//set text of pangolayout
@@ -217,8 +227,14 @@ void Dialog_Partition_Info::Display_Info()
 	Sector ptn_sectors = partition .get_sector_length() ;
 
 	Glib::ustring vgname = "" ;  //Also used in partition status message
-	if ( partition .filesystem == FS_LVM2_PV )
-		vgname = LVM2_PV_Info::get_vg_name( partition.get_path() );
+	if ( filesystem->filesystem == FS_LVM2_PV )
+		vgname = LVM2_PV_Info::get_vg_name( filesystem->get_path() );
+
+	bool filesystem_accessible = false;
+	if ( partition.filesystem != FS_LUKS || partition.busy )
+		// As long as this is not a LUKS encrypted partition which is closed the
+		// file system is accessible.
+		filesystem_accessible = true;
 
 	//initialize table top and bottom row number attach trackers
 	int top = 0     , bottom = 1 ;      //Left field & value pairs
@@ -247,49 +263,71 @@ void Dialog_Partition_Info::Display_Info()
 			1, 2,
 			top, bottom,
 			Gtk::FILL ) ;
-	table ->attach( * Utils::mk_label( Utils::get_filesystem_string( partition .filesystem ), true, false, true ),
-			2, 3,
-			top++, bottom++,
-			Gtk::FILL ) ;
+	if ( filesystem_accessible )
+	{
+		table->attach( *Utils::mk_label( Utils::get_filesystem_string( filesystem->filesystem ), true, false, true ),
+		               2, 3, top, bottom, Gtk::FILL );
+	}
+	top++, bottom++;
 
 	//label
-	if ( partition.type != GParted::TYPE_UNALLOCATED && partition.type != GParted::TYPE_EXTENDED )
+	if ( filesystem->type != TYPE_UNALLOCATED && filesystem->type != TYPE_EXTENDED )
 	{
 		table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("Label:") ) + "</b>"),
 				1, 2,
 				top, bottom,
 				Gtk::FILL) ;
-		table ->attach( * Utils::mk_label( partition.get_filesystem_label(), true, false, true ),
-				2, 3,
-				top++, bottom++,
-				Gtk::FILL) ;
+		if ( filesystem_accessible )
+		{
+			table->attach( *Utils::mk_label( filesystem->get_filesystem_label(), true, false, true ),
+			               2, 3, top, bottom, Gtk::FILL);
+		}
+		top++, bottom++;
 	}
 
-	//uuid
-	if ( partition.type != GParted::TYPE_UNALLOCATED && partition.type != GParted::TYPE_EXTENDED )
+	// file system uuid
+	if ( filesystem->type != TYPE_UNALLOCATED && filesystem->type != TYPE_EXTENDED )
 	{
 		table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("UUID:") ) + "</b>"),
 				1, 2,
 				top, bottom,
 				Gtk::FILL) ;
-		table ->attach( * Utils::mk_label( partition .uuid, true, false, true ),
-				2, 3,
-				top++, bottom++,
-				Gtk::FILL) ;
+		if ( filesystem_accessible )
+		{
+			table->attach( *Utils::mk_label( filesystem->uuid, true, false, true ),
+			               2, 3, top, bottom, Gtk::FILL);
+		}
+		top++, bottom++;
 	}
 
+	/* TO TRANSLATORS:   Open
+	 * means that the LUKS encryption is open and the encrypted data within is accessible.
+	 */
+	static Glib::ustring luks_open   = _("Open");
+	/* TO TRANSLATORS:   Closed
+	 * means that the LUKS encryption is closed and the encrypted data within is not accessible.
+	 */
+	static Glib::ustring luks_closed = _("Closed");
+
 	//status
-	if ( partition .type != GParted::TYPE_UNALLOCATED && partition .status != GParted::STAT_NEW )
+	if ( filesystem->type != TYPE_UNALLOCATED && filesystem->status != STAT_NEW )
 	{
 		//status
 		Glib::ustring str_temp ;
-		table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("Status:") ) + "</b>" ),
-				1, 2,
-				top, bottom,
-				Gtk::FILL ) ;
-		if ( partition .busy )
+		table->attach( *Utils::mk_label( "<b>" + Glib::ustring( _("Status:") ) + "</b>",
+		                                 true, false, false, 0.0 /* ALIGN_TOP */ ),
+		               1, 2, top, bottom, Gtk::FILL );
+		if ( ! filesystem_accessible )
 		{
-			if ( partition .type == GParted::TYPE_EXTENDED )
+			/* TO TRANSLATORS:   Not accessible (Encrypted)
+			 * means that the data in encrypted and hasn't been made
+			 * accessible by opening it with the passphrase.
+			 */
+			str_temp = _("Not accessible (Encrypted)");
+		}
+		else if ( filesystem->busy )
+		{
+			if ( filesystem->type == TYPE_EXTENDED )
 			{
 				/* TO TRANSLATORS:  Busy (At least one logical partition is mounted)
 				 * means that this extended partition contains at least one logical
@@ -297,10 +335,9 @@ void Dialog_Partition_Info::Display_Info()
 				 */
 				str_temp = _("Busy (At least one logical partition is mounted)") ;
 			}
-			else if (    partition .filesystem == FS_LINUX_SWAP
-			          || partition .filesystem == FS_LINUX_SWRAID
-			          || partition .filesystem == FS_LVM2_PV
-			         )
+			else if ( filesystem->filesystem == FS_LINUX_SWAP   ||
+			          filesystem->filesystem == FS_LINUX_SWRAID ||
+			          filesystem->filesystem == FS_LVM2_PV         )
 			{
 				/* TO TRANSLATORS:  Active
 				 * means that this linux swap, linux software raid partition, or
@@ -308,15 +345,23 @@ void Dialog_Partition_Info::Display_Info()
 				 */
 				str_temp = _("Active") ;
 			}
-			else if ( partition .get_mountpoints() .size() )
+			else if ( filesystem->filesystem == FS_LUKS )
+			{
+				// NOTE: LUKS within LUKS
+				// Only ever display LUKS information in the file system
+				// section when the contents of a LUKS encryption is
+				// another LUKS encryption!
+				str_temp = luks_open;
+			}
+			else if ( filesystem->get_mountpoints().size() )
 			{
 				str_temp = String::ucompose(
 						/* TO TRANSLATORS: looks like   Mounted on /mnt/mymountpoint */
 						_("Mounted on %1"),
-						Glib::build_path( ", ", partition .get_mountpoints() ) ) ;
+						Glib::build_path( ", ", filesystem->get_mountpoints() ) );
 			}
 		}
-		else if ( partition.type == GParted::TYPE_EXTENDED )
+		else if ( filesystem->type == TYPE_EXTENDED )
 		{
 			/* TO TRANSLATORS:  Not busy (There are no mounted logical partitions)
 			 * means that this extended partition contains no mounted or otherwise
@@ -324,9 +369,8 @@ void Dialog_Partition_Info::Display_Info()
 			 */
 			str_temp = _("Not busy (There are no mounted logical partitions)") ;
 		}
-		else if (    partition .filesystem == FS_LINUX_SWAP
-		          || partition .filesystem == FS_LINUX_SWRAID
-		        )
+		else if ( filesystem->filesystem == FS_LINUX_SWAP   ||
+		          filesystem->filesystem == FS_LINUX_SWRAID    )
 		{
 			/* TO TRANSLATORS:  Not active
 			 *  means that this linux swap or linux software raid partition
@@ -334,7 +378,12 @@ void Dialog_Partition_Info::Display_Info()
 			 */
 			str_temp = _("Not active") ;
 		}
-		else if ( partition .filesystem == FS_LVM2_PV )
+		else if ( filesystem->filesystem == FS_LUKS )
+		{
+			// NOTE: LUKS within LUKS
+			str_temp = luks_closed;
+		}
+		else if ( filesystem->filesystem == FS_LVM2_PV )
 		{
 			if ( vgname .empty() )
 				/* TO TRANSLATORS:  Not active (Not a member of any volume group)
@@ -366,11 +415,11 @@ void Dialog_Partition_Info::Display_Info()
 			str_temp = _("Not mounted") ;
 		}
 
-		table ->attach( * Utils::mk_label( str_temp, true, false, true ), 2, 3, top++, bottom++, Gtk::FILL ) ;
+		table->attach( *Utils::mk_label( str_temp, true, true, true ), 2, 3, top++, bottom++, Gtk::FILL );
 	}
 
 	//Optional, LVM2 Volume Group name
-	if ( partition .filesystem == FS_LVM2_PV )
+	if ( filesystem->filesystem == FS_LVM2_PV )
 	{
 		//Volume Group
 		table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("Volume Group:") ) + "</b>"),
@@ -380,18 +429,18 @@ void Dialog_Partition_Info::Display_Info()
 	}
 
 	//Optional, members of multi-device file systems
-	if (    partition .filesystem == FS_LVM2_PV
-	     || partition .filesystem == FS_BTRFS   )
+	if ( filesystem->filesystem == FS_LVM2_PV ||
+	     filesystem->filesystem == FS_BTRFS      )
 	{
 		//Members
 		table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("Members:") ) + "</b>", true, false, false, 0.0 /* ALIGN_TOP */ ),
 		                1, 2, top, bottom, Gtk::FILL ) ;
 
 		std::vector<Glib::ustring> members ;
-		switch ( partition .filesystem )
+		switch ( filesystem->filesystem )
 		{
 			case FS_BTRFS:
-				members = btrfs::get_members( partition .get_path() ) ;
+				members = btrfs::get_members( filesystem->get_path() );
 				break ;
 			case FS_LVM2_PV:
 				if ( ! vgname .empty() )
@@ -401,37 +450,19 @@ void Dialog_Partition_Info::Display_Info()
 				break ;
 		}
 
-		Glib::ustring members_str = "" ;
-		if ( ! members .empty() )
-		{
-			for ( unsigned int i = 0 ; i < members .size() ; i ++ )
-			{
-				if ( i > 0 )
-					members_str += "\n" ;
-				members_str += members[i] ;
-			}
-		}
-		table ->attach( * Utils::mk_label( members_str, true, false, true, 0.0 /* ALIGN_TOP */ ), 2, 3, top++, bottom++, Gtk::FILL ) ;
+		table->attach( *Utils::mk_label( Glib::build_path( "\n", members ), true, false, true ),
+		               2, 3, top++, bottom++, Gtk::FILL );
 	}
 
-	if ( partition.filesystem == FS_LVM2_PV )
+	if ( filesystem->filesystem == FS_LVM2_PV )
 	{
 		//Logical Volumes
 		table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("Logical Volumes:") ) + "</b>", true, false, false, 0.0 /* ALIGN_TOP */ ),
 		                1, 2, top, bottom, Gtk::FILL );
 
-		std::vector<Glib::ustring> lvs;
-		lvs = LVM2_PV_Info::get_vg_lvs( vgname );
-
-		Glib::ustring lvs_str = "";
-		for ( unsigned int i = 0 ; i < lvs .size() ; i ++ )
-		{
-			if ( i > 0 )
-				lvs_str += "\n";
-			lvs_str += lvs[i];
-		}
-
-		table ->attach( * Utils::mk_label( lvs_str, true, false, true, 0.0 /* ALIGN_TOP */ ), 2, 3, top++, bottom++, Gtk::FILL );
+		std::vector<Glib::ustring> lvs = LVM2_PV_Info::get_vg_lvs( vgname );
+		table->attach( *Utils::mk_label( Glib::build_path( "\n", lvs ), true, false, true ),
+		               2, 3, top++, bottom++, Gtk::FILL );
 	}
 
 	//Right field & value pair area
@@ -505,6 +536,47 @@ void Dialog_Partition_Info::Display_Info()
 	//one blank line
 	table ->attach( * Utils::mk_label( "" ), 0, 6, top++, bottom++, Gtk::FILL ) ;
 
+	if ( partition.filesystem == FS_LUKS )
+	{
+		// ENCRYPTION DETAIL SECTION
+		// encryption headline
+		table->attach( *Utils::mk_label( "<b>" + Glib::ustring( _("Encryption") ) + "</b>" ),
+		               0, 6, top++, bottom++, Gtk::FILL );
+
+		// Encryption
+		table->attach( *Utils::mk_label( "<b>" + Glib::ustring( _("Encryption:") ) + "</b>"),
+		               1, 2, top, bottom, Gtk::FILL);
+		table->attach( *Utils::mk_label( Utils::get_filesystem_string( partition.filesystem ), true, false, true ),
+		               2, 3, top++, bottom++, Gtk::FILL);
+
+		// LUKS path
+		table->attach( *Utils::mk_label( "<b>" + Glib::ustring( _("Path:") ) + "</b>"),
+		               1, 2, top, bottom, Gtk::FILL);
+		if ( partition.busy )
+			table->attach( *Utils::mk_label( partition.get_mountpoint(), true, false, true ),
+			               2, 3, top, bottom, Gtk::FILL);
+		top++, bottom++;
+
+		// LUKS uuid
+		table->attach( *Utils::mk_label( "<b>" + Glib::ustring( _("UUID:") ) + "</b>"),
+		               1, 2, top, bottom, Gtk::FILL);
+		table->attach( *Utils::mk_label( partition.uuid, true, false, true ),
+		               2, 3, top++, bottom++, Gtk::FILL);
+
+		// LUKS status
+		table->attach( *Utils::mk_label( "<b>" + Glib::ustring( _("Status:") ) + "</b>" ),
+		               1, 2, top, bottom, Gtk::FILL );
+		Glib::ustring str_temp;
+		if ( partition.busy )
+			str_temp = luks_open;
+		else
+			str_temp = luks_closed;
+		table->attach( *Utils::mk_label( str_temp, true, false, true ), 2, 3, top++, bottom++, Gtk::FILL );
+
+		// one blank line
+		table->attach( *Utils::mk_label( "" ), 0, 6, top++, bottom++, Gtk::FILL );
+	}
+
 	//PARTITION DETAIL SECTION
 	//partition headline
 	table ->attach( * Utils::mk_label( "<b>" + Glib::ustring( _("Partition") ) + "</b>" ),
@@ -522,10 +594,10 @@ void Dialog_Partition_Info::Display_Info()
 			1, 2,
 			top, bottom,
 			Gtk::FILL ) ;
-	table ->attach( * Utils::mk_label( Glib::build_path( "\n", partition .get_paths() ), true, false, true ),
-			2, 3,
-			top++, bottom++,
-			Gtk::FILL ) ;
+	table->attach( * Utils::mk_label( partition.get_path(), true, false, true ),
+	               2, 3,
+	               top++, bottom++,
+	               Gtk::FILL );
 
 	if ( partition .type != GParted::TYPE_UNALLOCATED && partition .status != GParted::STAT_NEW )
 	{
