@@ -15,8 +15,13 @@
  */
 
 #include "../include/Proc_Partitions_Info.h"
+#include "../include/BlockSpecial.h"
+#include "../include/Utils.h"
 
+#include <glibmm/ustring.h>
+#include <vector>
 #include <fstream>
+#include <stdio.h>
 
 namespace GParted
 {
@@ -24,88 +29,77 @@ namespace GParted
 //Initialize static data elements
 bool Proc_Partitions_Info::proc_partitions_info_cache_initialized = false ;
 std::vector<Glib::ustring> Proc_Partitions_Info::device_paths_cache ;
-std::map< Glib::ustring, Glib::ustring > Proc_Partitions_Info::alternate_paths_cache ;
 
-Proc_Partitions_Info::Proc_Partitions_Info()
+void Proc_Partitions_Info::load_cache()
+{
+	load_proc_partitions_info_cache();
+	proc_partitions_info_cache_initialized = true;
+}
+
+const std::vector<Glib::ustring> & Proc_Partitions_Info::get_device_paths()
+{
+	initialize_if_required();
+	return device_paths_cache;
+}
+
+// Private Methods
+
+void Proc_Partitions_Info::initialize_if_required()
 {
-	//Ensure that cache has been loaded at least once
 	if ( ! proc_partitions_info_cache_initialized )
 	{
-		proc_partitions_info_cache_initialized = true ;
-		load_proc_partitions_info_cache() ;
+		load_proc_partitions_info_cache();
+		proc_partitions_info_cache_initialized = true;
 	}
 }
 
-Proc_Partitions_Info::Proc_Partitions_Info( bool do_refresh )
-{
-	//Ensure that cache has been loaded at least once
-	if ( ! proc_partitions_info_cache_initialized )
-	{
-		proc_partitions_info_cache_initialized = true ;
-		if ( do_refresh == false )
-			load_proc_partitions_info_cache() ;
-	}
-
-	if ( do_refresh )
-		load_proc_partitions_info_cache() ;
-}
-
-Proc_Partitions_Info::~Proc_Partitions_Info()
-{
-}
-
-std::vector<Glib::ustring> Proc_Partitions_Info::get_device_paths()
-{
-	return device_paths_cache ;
-}
-
-std::vector<Glib::ustring> Proc_Partitions_Info::get_alternate_paths( const Glib::ustring & path ) 
-{
-	std::vector<Glib::ustring> paths ;
-	std::map< Glib::ustring, Glib::ustring >::iterator iter ;
-
-	iter = alternate_paths_cache .find( path ) ;
-	if ( iter != alternate_paths_cache .end() )
-		paths .push_back( iter ->second ) ;
-
-	return paths ;
-}
-
-//Private Methods
 void Proc_Partitions_Info::load_proc_partitions_info_cache()
 {
-	alternate_paths_cache .clear();
 	device_paths_cache .clear() ;
 
-	//Initialize alternate_paths
 	std::ifstream proc_partitions( "/proc/partitions" ) ;
 	if ( proc_partitions )
 	{
 		std::string line ;
-		std::string device ;
-		char c_str[4096+1] ;
+		Glib::ustring device;
 
 		while ( getline( proc_partitions, line ) )
 		{
-			//Build cache of disk devices.
+			// Pre-populate BlockSpecial cache with major, minor numbers of
+			// all names found from /proc/partitions.
+			Glib::ustring name = Utils::regexp_label( line,
+			        "^[[:blank:]]*[[:digit:]]+[[:blank:]]+[[:digit:]]+[[:blank:]]+[[:digit:]]+[[:blank:]]+([[:graph:]]+)$" );
+			if ( name == "" )
+				continue;
+			unsigned long maj;
+			unsigned long min;
+			if ( sscanf( line.c_str(), "%lu %lu", &maj, &min ) != 2 )
+				continue;
+			BlockSpecial::register_block_special( "/dev/" + name, maj, min );
+
+			// Recognise only whole disk device names, excluding partitions,
+			// from /proc/partitions and save in this cache.
+
 			//Whole disk devices are the ones we want.
 			//Device names without a trailing digit refer to the whole disk.
-			device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+([^0-9]+)$") ;
+			device = Utils::regexp_label( name, "^([^0-9]+)$" );
+
 			//Recognize /dev/md* devices (Linux software RAID - mdadm).
 			//E.g., device = /dev/md127, partition = /dev/md127p1
 			if ( device == "" )
-				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(md[0-9]+)$") ;
+				device = Utils::regexp_label( name, "^(md[0-9]+)$" );
+
 			//Recognize /dev/mmcblk* devices.
 			//E.g., device = /dev/mmcblk0, partition = /dev/mmcblk0p1
 			if ( device == "" )
-				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(mmcblk[0-9]+)$") ;
+				device = Utils::regexp_label( name, "^(mmcblk[0-9]+)$" );
 
 			// Recognise /dev/nvme*n* devices
 			// (Non-Volatile Memory Express devices.  SSD type devices which
 			// plug directly into PCIe sockets).
 			// E.g., device = /dev/nvme0n1, partition = /dev/nvme0n1p1
 			if ( device == "" )
-				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(nvme[0-9]+n[0-9]+)$");
+				device = Utils::regexp_label( name, "^(nvme[0-9]+n[0-9]+)$" );
 
 			//Device names that end with a #[^p]# are HP Smart Array Devices (disks)
 			//  E.g., device = /dev/cciss/c0d0, partition = /dev/cciss/c0d0p1
@@ -117,33 +111,13 @@ void Proc_Partitions_Info::load_proc_partitions_info_cache()
 			//  E.g., device = /dev/rd/c0d0, partition = /dev/rd/c0d0p1
 			//  (linux-x.y.z/Documentation/blockdev/README.DAC960)
 			if ( device == "" )
-				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+([a-z]+/c[0-9]+d[0-9]+)$") ;
+				device = Utils::regexp_label( name, "^([a-z]+/c[0-9]+d[0-9]+)$" );
+
 			if ( device != "" )
 			{
 				//add potential device to the list
-				device = "/dev/" + device;
-				device_paths_cache .push_back( device ) ;
+				device_paths_cache.push_back( "/dev/" + device );
 			}
-
-			//Build cache of potential alternate paths
-			if ( sscanf( line .c_str(), "%*d %*d %*d %4096s", c_str ) == 1 )
-			{
-				line = "/dev/" ; 
-				line += c_str ;
-
-				//FIXME: it seems realpath is very unsafe to use (manpage)...
-				if (   file_test( line, Glib::FILE_TEST_EXISTS )
-				    && realpath( line .c_str(), c_str )
-				    //&& line != c_str
-				   )
-				{
-					//Because we can make no assumption about which path libparted will
-					//detect, we add all combinations.
-					alternate_paths_cache[ c_str ] = line ;
-					alternate_paths_cache[ line ] = c_str ;
-				}
-			}
-
 		}
 		proc_partitions .close() ;
 	}
