@@ -16,6 +16,14 @@
  */
  
 #include "../include/TreeView_Detail.h"
+#include "../include/Partition.h"
+#include "../include/PartitionLUKS.h"
+#include "../include/PartitionVector.h"
+
+#include <vector>
+#include <gtkmm/cellrenderer.h>
+#include <gtkmm/cellrenderertext.h>
+#include <pangomm/layout.h>
 
 namespace GParted
 { 
@@ -40,37 +48,25 @@ TreeView_Detail::TreeView_Detail()
 	append_column( _("Used"), treeview_detail_columns .used );
 	append_column( _("Unused"), treeview_detail_columns .unused );
 	append_column( _("Flags"), treeview_detail_columns .flags );
-	
-	//icons
+
+	// Tree view column "Partition"; add icon cells.
 	get_column( 0 ) ->pack_start( treeview_detail_columns .icon2, false );
 	get_column( 0 ) ->pack_start( treeview_detail_columns .icon1, false );
 
-	//PARTITION
-	//colored text in Partition column 
-	Gtk::CellRendererText *cell_renderer_text = 
-		dynamic_cast<Gtk::CellRendererText*>( get_column( 0 ) ->get_first_cell_renderer() );
-	get_column( 0 ) ->add_attribute( cell_renderer_text ->property_foreground(), 
-					 treeview_detail_columns .text_color );
-	
-	//FILE SYSTEM
-	//file system text
+	// Tree view column "File System"; add file system text cell.
 	get_column( 2 )->pack_start( treeview_detail_columns.filesystem, true );
-
-	//colored text in File System column 
-	std::vector<Gtk::CellRenderer*> renderers = get_column( 2 )->get_cell_renderers();
-	cell_renderer_text = dynamic_cast<Gtk::CellRendererText*>( renderers .back() ) ;
-	get_column( 2 )->add_attribute( cell_renderer_text->property_foreground(),
-	                                treeview_detail_columns.text_color );
-
-	//pixbuf and text are both left aligned
+	// Color pixbuf cell is left aligned.
 	get_column( 2 )->get_first_cell_renderer()->property_xalign() = Gtk::ALIGN_LEFT;
+	// File system text cell is left aligned.
+	std::vector<Gtk::CellRenderer*> renderers = get_column( 2 )->get_cell_renderers();
+	Gtk::CellRendererText *cell_renderer_text = dynamic_cast<Gtk::CellRendererText*>( renderers.back() );
 	cell_renderer_text ->property_xalign() = Gtk::ALIGN_LEFT ;
-	
-	//MOUNT POINT
-	//colored text in mount point column 
-	cell_renderer_text = dynamic_cast<Gtk::CellRendererText*>( get_column( 3 )->get_first_cell_renderer() );
-	get_column( 3 )->add_attribute( cell_renderer_text->property_foreground(),
-	                                treeview_detail_columns.mount_text_color );
+
+	// Tree view column "Mount Point", make column resizable and show too wide text
+	// with ellipsis.
+	get_column( 3 )->set_resizable( true );
+	cell_renderer_text = dynamic_cast<Gtk::CellRendererText *>( get_column( 3 )->get_first_cell_renderer() );
+	cell_renderer_text->property_ellipsize() = Pango::ELLIPSIZE_END;
 
 	//set alignment of numeric columns to right
 	for( short t = 5 ; t < 8 ; t++ )
@@ -84,7 +80,7 @@ TreeView_Detail::TreeView_Detail()
 	}
 }
 
-void TreeView_Detail::load_partitions( const std::vector<Partition> & partitions ) 
+void TreeView_Detail::load_partitions( const PartitionVector & partitions )
 {
 	bool mountpoints = false, labels = false, names = false;
 	treestore_detail ->clear() ;
@@ -111,7 +107,7 @@ void TreeView_Detail::clear()
 	treestore_detail ->clear() ;
 }
 
-void TreeView_Detail::load_partitions( const std::vector<Partition> & partitions,
+void TreeView_Detail::load_partitions( const PartitionVector & partitions,
                                        bool & mountpoints,
                                        bool & labels,
                                        bool & names,
@@ -164,7 +160,7 @@ void TreeView_Detail::create_row( const Gtk::TreeRow & treerow, const Partition
 		treerow[ treeview_detail_columns .icon1 ] = 
 			render_icon( Gtk::Stock::DIALOG_AUTHENTICATION, Gtk::ICON_SIZE_BUTTON );
 	
-	if ( partition .messages .size() > 0 )
+	if ( partition.have_messages() > 0 )
 	{
 		if ( ! static_cast< Glib::RefPtr<Gdk::Pixbuf> >( treerow[ treeview_detail_columns .icon1 ] )  )
 			treerow[ treeview_detail_columns .icon1 ] = 
@@ -184,13 +180,37 @@ void TreeView_Detail::create_row( const Gtk::TreeRow & treerow, const Partition
 	// name
 	treerow[treeview_detail_columns.name] = partition.name;
 
-	treerow[ treeview_detail_columns .color ] = Utils::get_color_as_pixbuf( partition .filesystem, 16, 16 ) ; 
+	if ( partition.filesystem == FS_LUKS && partition.busy )
+	{
+		const Partition & encrypted = dynamic_cast<const PartitionLUKS *>( &partition )->get_encrypted();
 
-	treerow[ treeview_detail_columns .filesystem ] = 
-		Utils::get_filesystem_string( partition .filesystem ) ;
-	
-	//mount point
-	treerow[ treeview_detail_columns .mountpoint ] = Glib::build_path( ", ", partition .get_mountpoints() ) ;
+		// file system
+		treerow[treeview_detail_columns.color] = Utils::get_color_as_pixbuf( encrypted.filesystem, 16, 16 );
+		/* TO TRANSLATORS: means that this is an encrypted file system */
+		treerow[treeview_detail_columns.filesystem] = "[" + Glib::ustring( _("Encrypted") ) + "] " +
+		                                              Utils::get_filesystem_string( encrypted.filesystem );
+
+		// mount point
+		treerow[treeview_detail_columns.mountpoint] = Glib::build_path( ", ", encrypted.get_mountpoints() );
+	}
+	else if ( partition.filesystem == FS_LUKS && ! partition.busy )
+	{
+		// file system
+		treerow[treeview_detail_columns.color] = Utils::get_color_as_pixbuf( partition.filesystem, 16, 16 );
+		treerow[treeview_detail_columns.filesystem] = "[" + Glib::ustring( _("Encrypted") ) + "]";
+
+		// mount point
+		treerow[treeview_detail_columns.mountpoint] = Glib::build_path( ", ", partition.get_mountpoints() );
+	}
+	else
+	{
+		// file system
+		treerow[treeview_detail_columns.color] = Utils::get_color_as_pixbuf( partition.filesystem, 16, 16 );
+		treerow[treeview_detail_columns.filesystem] = Utils::get_filesystem_string( partition.filesystem );
+
+		// mount point
+		treerow[treeview_detail_columns.mountpoint] = Glib::build_path( ", ", partition.get_mountpoints() );
+	}
 
 	//label
 	treerow[ treeview_detail_columns .label ] = partition.get_filesystem_label();
