@@ -31,6 +31,7 @@
 #include <uuid/uuid.h>
 #include <cerrno>
 #include <sys/statvfs.h>
+#include <glibmm/ustring.h>
 #include <gtkmm/main.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -39,9 +40,17 @@
 namespace GParted
 {
 
+const Glib::ustring DEV_MAPPER_PATH = "/dev/mapper/";
+
 Sector Utils::round( double double_value )
 {
-	 return static_cast<Sector>( double_value + 0.5 ) ;
+	// Reference:
+	//     How can I convert a floating-point value to an integer in C?
+	//     https://www.cs.tut.fi/~jkorpela/round.html
+	if ( double_value >= 0.0 )
+		return static_cast<Sector>( double_value + 0.5 );
+	else
+		return static_cast<Sector>( double_value - 0.5 );
 }
 
 Gtk::Label * Utils::mk_label( const Glib::ustring & text
@@ -106,6 +115,7 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 		case FS_UNUSED		: return "#FFFFFF" ;	//white
 		case FS_LVM2_PV		: return "#B39169" ;	//face skin dark
 		case FS_BITLOCKER	: return "#494066" ;	//purple shadow
+		case FS_GRUB2_CORE_IMG	: return "#666666" ;	//~ dark gray
 		case FS_LUKS		: return "#625B81" ;	//purple dark
 		case FS_LINUX_SWRAID   	: return "#5A4733" ;	// ~ dark brown
 		case FS_LINUX_SWSUSPEND	: return "#884631" ;	//red dark
@@ -282,7 +292,8 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 		case FS_UNUSED		: return _("unused") ;
 		case FS_LVM2_PV		: return "lvm2 pv" ;
 		case FS_BITLOCKER	: return "bitlocker" ;
-		case FS_LUKS		: return "crypt-luks" ;
+		case FS_GRUB2_CORE_IMG	: return "grub2 core.img";
+		case FS_LUKS		: return "luks";
 		case FS_LINUX_SWRAID	: return "linux-raid" ;
 		case FS_LINUX_SWSUSPEND	: return "linux-suspend" ;
 		case FS_REFS		: return "refs" ;
@@ -335,6 +346,7 @@ Glib::ustring Utils::get_filesystem_software( FILESYSTEM filesystem )
 		case FS_JFS         : return "jfsutils" ;
 		case FS_LINUX_SWAP  : return "util-linux" ;
 		case FS_LVM2_PV     : return "lvm2" ;
+		case FS_LUKS        : return "dmsetup";
 		case FS_NILFS2      : return "nilfs-utils" ;
 		case FS_NTFS        : return "ntfs-3g / ntfsprogs" ;
 		case FS_REISER4     : return "reiser4progs" ;
@@ -437,6 +449,12 @@ Glib::ustring Utils::format_time( std::time_t seconds )
 {
 	Glib::ustring time ;
 
+	if ( seconds < 0 )
+	{
+		time = "-";
+		seconds = -seconds;
+	}
+
 	int unit = static_cast<int>( seconds / 3600 ) ;
 	if ( unit < 10 )
 		time += "0" ;
@@ -667,6 +685,16 @@ Glib::ustring Utils::trim( const Glib::ustring & src, const Glib::ustring & c /*
 	return src.substr(p1, (p2-p1)+1);
 }
 
+// Return portion of string after the last carriage return character or
+// the whole string when there is no carriage return character.
+Glib::ustring Utils::last_line( const Glib::ustring & src )
+{
+	Glib::ustring::size_type p = src.find_last_of( '\n' );
+	if ( p == Glib::ustring::npos )
+		return src;
+	return src.substr( p+1 );
+}
+
 Glib::ustring Utils::get_lang()
 {
 	//Extract base language from string that may look like "en_CA.UTF-8"
