@@ -43,6 +43,42 @@ FS ext2::get_filesystem_support()
 		fs .create = FS::EXTERNAL ;
 		fs .create_with_label = FS::EXTERNAL ;
 
+		// Determine mkfs.ext4 version specific capabilities.
+		bool have_64bit_feature = false;
+		force_auto_64bit = false;
+		if ( specific_type == FS_EXT4 )
+		{
+			Utils::execute_command( mkfs_cmd + " -V", output, error, true );
+			int mke4fs_major_ver = 0;
+			int mke4fs_minor_ver = 0;
+			int mke4fs_patch_ver = 0;
+			if ( sscanf( error.c_str(), "mke%*[24]fs %d.%d.%d",
+			             &mke4fs_major_ver, &mke4fs_minor_ver, &mke4fs_patch_ver ) >= 2 )
+			{
+				// Ext4 64bit feature was added in e2fsprogs 1.42, but
+				// only enable large volumes from 1.42.9 when a large
+				// number of 64bit bugs were fixed.
+				// *   Release notes, E2fsprogs 1.42 (November 29, 2011)
+				//     http://e2fsprogs.sourceforge.net/e2fsprogs-release.html#1.42
+				// *   Release notes, E2fsprogs 1.42.9 (December 28, 2013)
+				//     http://e2fsprogs.sourceforge.net/e2fsprogs-release.html#1.42.9
+				have_64bit_feature =    ( mke4fs_major_ver > 1 )
+				                     || ( mke4fs_major_ver == 1 && mke4fs_minor_ver > 42 )
+				                     || ( mke4fs_major_ver == 1 && mke4fs_minor_ver == 42 && mke4fs_patch_ver >= 9 );
+
+				// (#766910) E2fsprogs 1.43 creates 64bit ext4 file
+				// systems by default.  RHEL/CentOS 7 configured e2fsprogs
+				// 1.42.9 to create 64bit ext4 file systems by default.
+				// Theoretically this can be done when 64bit feature was
+				// added in e2fsprogs 1.42.  GParted will re-implement the
+				// removed mke2fs.conf(5) auto_64-bit_support option to
+				// avoid the issues with multiple boot loaders not working
+				// with 64bit ext4 file systems.
+				force_auto_64bit =    ( mke4fs_major_ver > 1 )
+				                   || ( mke4fs_major_ver == 1 && mke4fs_minor_ver >= 42 );
+			}
+		}
+
 		if ( specific_type == FS_EXT4 && ! Glib::find_program_in_path( "dumpe4fs" ).empty() )
 			dump_cmd = "dumpe4fs";
 		else if ( ! Glib::find_program_in_path( "dumpe2fs" ).empty() )
@@ -114,6 +150,19 @@ FS ext2::get_filesystem_support()
 		if ( specific_type != FS_EXT2 && Utils::kernel_version_at_least( 3, 6, 0 ) )
 			fs .online_grow = fs .grow ;
 #endif
+
+		// Maximum size of an ext2/3/4 volume is 2^32 - 1 blocks, except for ext4
+		// with 64bit feature.  That is just under 16 TiB with a 4K block size.
+		// *   Ext4 Disk Layout, Blocks
+		//     https://ext4.wiki.kernel.org/index.php/Ext4_Disk_Layout#Blocks
+		// FIXME: Rounding down to whole MiB here should not be necessary.  The
+		// Copy, New and Resize/Move dialogs should limit FS correctly without
+		// this.  See bug #766910 comment #12 onwards for further discussion.
+		//     https://bugzilla.gnome.org/show_bug.cgi?id=766910#c12
+		if ( specific_type == FS_EXT2                             ||
+		     specific_type == FS_EXT3                             ||
+		     ( specific_type == FS_EXT4 && ! have_64bit_feature )    )
+			fs.MAX = Utils::floor_size( 16 * TEBIBYTE - 4 * KIBIBYTE, MEBIBYTE );
 	}
 
 	return fs ;
@@ -130,12 +179,12 @@ void ext2::set_used_sectors( Partition & partition )
 	{
 		Glib::ustring::size_type index = output.find( "Block count:" );
 		if ( index >= output .length() ||
-		     sscanf( output.substr( index ) .c_str(), "Block count: %Ld", &T ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "Block count: %lld", &T ) != 1 )
 			T = -1 ;
 
 		index = output .find( "Block size:" ) ;
 		if ( index >= output.length() || 
-		     sscanf( output.substr( index ) .c_str(), "Block size: %Ld", &S ) != 1 )  
+		     sscanf( output.substr( index ).c_str(), "Block size: %lld", &S ) != 1 )
 			S = -1 ;
 
 		if ( T > -1 && S > -1 )
@@ -158,7 +207,7 @@ void ext2::set_used_sectors( Partition & partition )
 		{
 			index = output .find( "Free blocks:" ) ;
 			if ( index >= output .length() ||
-			     sscanf( output.substr( index ) .c_str(), "Free blocks: %Ld", &N ) != 1 )
+			     sscanf( output.substr( index ).c_str(), "Free blocks: %lld", &N ) != 1 )
 				N = -1 ;
 
 			if ( N > -1 && S > -1 )
@@ -228,8 +277,19 @@ bool ext2::write_uuid( const Partition & partition, OperationDetail & operationd
 
 bool ext2::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
-	return ! execute_command( mkfs_cmd + " -F -L \"" + new_partition.get_filesystem_label() + "\" " +
-	                          new_partition.get_path(),
+	Glib::ustring features;
+	if ( force_auto_64bit )
+	{
+		// (#766910) Manually implement mke2fs.conf(5) auto_64-bit_support option
+		// by setting or clearing the 64bit feature on the command line depending
+		// of the partition size.
+		if ( new_partition.get_byte_length() >= 16 * TEBIBYTE )
+			features = " -O 64bit";
+		else
+			features = " -O ^64bit";
+	}
+	return ! execute_command( mkfs_cmd + " -F" + features +
+	                          " -L \"" + new_partition.get_filesystem_label() + "\" " + new_partition.get_path(),
 	                          operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE|EXEC_PROGRESS_STDOUT,
 	                          static_cast<StreamSlot>( sigc::mem_fun( *this, &ext2::create_progress ) ) );
 }
@@ -322,7 +382,7 @@ void ext2::create_progress( OperationDetail *operationdetail )
 	Glib::ustring line = Utils::last_line( output );
 	// Text progress on the LAST LINE looks like "Writing inode tables:  105/1600"
 	long long progress, target;
-	if ( sscanf( line.c_str(), "Writing inode tables: %Ld/%Ld", &progress, &target ) == 2 )
+	if ( sscanf( line.c_str(), "Writing inode tables: %lld/%lld", &progress, &target ) == 2 )
 	{
 		operationdetail->run_progressbar( (double)progress, (double)target );
 	}
@@ -362,7 +422,7 @@ void ext2::copy_progress( OperationDetail *operationdetail )
 	Glib::ustring line = Utils::last_line( error );
 	// Text progress on the LAST LINE of STDERR looks like "Copying 146483 / 258033 blocks ..."
 	long long progress, target;
-	if ( sscanf( line.c_str(), "Copying %Ld / %Ld blocks", &progress, &target ) == 2 )
+	if ( sscanf( line.c_str(), "Copying %lld / %lld blocks", &progress, &target ) == 2 )
 	{
 		operationdetail->run_progressbar( (double)(progress * fs_block_size),
 		                                  (double)(target * fs_block_size),
