@@ -17,6 +17,7 @@
  
  
 #include "../include/fat16.h"
+#include "../include/Partition.h"
 
 /*****
 //For some reason unknown, this works without these include statements.
@@ -59,8 +60,7 @@ const Glib::ustring fat16::get_custom_text( CUSTOM_TEXT ttype, int index ) const
 
 FS fat16::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = specific_type ;
+	FS fs( specific_type );
 
 	// hack to disable silly mtools warnings
 	setenv( "MTOOLS_SKIP_CHECK", "1", 0 );
@@ -120,6 +120,11 @@ FS fat16::get_filesystem_support()
 	else //FS_FAT32
 	{
 		fs .MIN = 33 * MEBIBYTE ; //Smaller file systems will cause windows scandisk to fail.
+
+		// Maximum FAT32 volume size with 512 byte sectors is 2 TiB.
+		// *   Wikipedia: File Allocation Table / FAT32
+		//     https://en.wikipedia.org/wiki/File_Allocation_Table#FAT32
+		fs.MAX = 2 * TEBIBYTE;
 	}
 
 	return fs ;
@@ -131,13 +136,13 @@ void fat16::set_used_sectors( Partition & partition )
 	if ( exit_status == 0 || exit_status == 1 )
 	{
 		//total file system size in logical sectors
-		index = output .rfind( "\n", output .find( "sectors total" ) ) +1 ;
-		if ( index >= output .length() || sscanf( output .substr( index ) .c_str(), "%Ld", &T ) != 1 )
+		Glib::ustring::size_type index = output.rfind( "\n", output.find( "sectors total" ) ) + 1;
+		if ( index >= output.length() || sscanf( output.substr( index ).c_str(), "%lld", &T ) != 1 )
 			T = -1 ;
 
 		//bytes per logical sector
 		index = output .rfind( "\n", output .find( "bytes per logical sector" ) ) +1 ;
-		if ( index >= output .length() || sscanf( output .substr( index ) .c_str(), "%Ld", &S ) != 1 )
+		if ( index >= output.length() || sscanf( output.substr( index ).c_str(), "%lld", &S ) != 1 )
 			S = -1 ;
 
 		if ( T > -1 && S > -1 )
@@ -145,18 +150,21 @@ void fat16::set_used_sectors( Partition & partition )
 
 		//free clusters
 		index = output .find( ",", output .find( partition .get_path() ) + partition .get_path() .length() ) +1 ;
-		if ( index < output .length() && sscanf( output .substr( index ) .c_str(), "%Ld/%Ld", &S, &N ) == 2 ) 
+		if ( index < output.length() && sscanf( output.substr( index ).c_str(), "%lld/%lld", &S, &N ) == 2 )
 			N -= S ;
 		else
 			N = -1 ;
 
 		//bytes per cluster
 		index = output .rfind( "\n", output .find( "bytes per cluster" ) ) +1 ;
-		if ( index >= output .length() || sscanf( output .substr( index ) .c_str(), "%Ld", &S ) != 1 )
+		if ( index >= output.length() || sscanf( output.substr( index ).c_str(), "%lld", &S ) != 1 )
 			S = -1 ;
 	
 		if ( N > -1 && S > -1 )
+		{
 			N = Utils::round( N * ( S / double(partition .sector_size) ) ) ;
+			partition.fs_block_size = S;
+		}
 
 		if ( T > -1 && N > -1 )
 			partition .set_sector_usage( T, N ) ;
@@ -164,10 +172,10 @@ void fat16::set_used_sectors( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -180,10 +188,10 @@ void fat16::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -212,10 +220,10 @@ void fat16::read_uuid( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
