@@ -17,14 +17,14 @@
  
  
 #include "../include/hfs.h"
+#include "../include/Partition.h"
 
 namespace GParted
 {
 
 FS hfs::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = GParted::FS_HFS ;
+	FS fs( FS_HFS );
 
 	fs .busy = FS::GPARTED ;
 
@@ -42,9 +42,6 @@ FS hfs::get_filesystem_support()
 	if ( ! Glib::find_program_in_path( "hfsck" ) .empty() )
 		fs .check = FS::EXTERNAL ;
 
-	if ( ! Glib::find_program_in_path( "vol_id" ) .empty() )
-		fs .read_label = FS::EXTERNAL ;
-
 	fs .copy = GParted::FS::GPARTED ;
 	fs .move = GParted::FS::GPARTED ;
 	fs .online_read = FS::GPARTED ;
@@ -54,27 +51,6 @@ FS hfs::get_filesystem_support()
 	return fs ;
 }
 
-void hfs::read_label( Partition & partition )
-{
-	if ( ! Utils::execute_command( "vol_id " + partition .get_path(), output, error, true ) )
-	{
-		Glib::ustring label = Utils::regexp_label( output, "ID_FS_LABEL=([^\n]*)" ) ;
-		//FIXME: find a better way to see if label is empty.. imagine someone uses 'untitled' as label.... ;)
-		if ( label != "untitled" )
-			partition.set_filesystem_label( label );
-		else
-			partition.set_filesystem_label( "" );
-	}
-	else
-	{
-		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
-		
-		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
-	}
-}
-
 bool hfs::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
 	Glib::ustring cmd = "";
