@@ -17,14 +17,14 @@
  
  
 #include "../include/jfs.h"
+#include "../include/Partition.h"
 
 namespace GParted
 {
 
 FS jfs::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = GParted::FS_JFS ;
+	FS fs( FS_JFS );
 
 	fs .busy = FS::GPARTED ;
 
@@ -79,21 +79,21 @@ void jfs::set_used_sectors( Partition & partition )
 	if ( ! Utils::execute_command( "sh -c 'echo dm | jfs_debugfs " + partition.get_path() + "'", output, error, true ) )
 	{
 		//blocksize
-		index = output .find( "Block Size:" ) ;
+		Glib::ustring::size_type index = output.find( "Block Size:" );
 		if ( index >= output .length() || 
-		     sscanf( output .substr( index ) .c_str(), "Block Size: %Ld", &S ) != 1 ) 
+		     sscanf( output.substr( index ).c_str(), "Block Size: %lld", &S ) != 1 )
 			S = -1 ;
 
 		//total blocks
 		index = output .find( "dn_mapsize:" ) ;
 		if ( index >= output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "dn_mapsize: %Lx", &T ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "dn_mapsize: %llx", &T ) != 1 )
 			T = -1 ;
 
 		//free blocks
 		index = output .find( "dn_nfree:" ) ;
 		if ( index >= output .length() || 
-		     sscanf( output .substr( index ) .c_str(), "dn_nfree: %Lx", &N ) != 1 ) 
+		     sscanf( output.substr( index ).c_str(), "dn_nfree: %llx", &N ) != 1 )
 			N = -1 ;
 
 		if ( T > -1 && N > -1 && S > -1 )
@@ -101,15 +101,16 @@ void jfs::set_used_sectors( Partition & partition )
 			T = Utils::round( T * ( S / double(partition .sector_size) ) ) ;
 			N = Utils::round( N * ( S / double(partition .sector_size) ) ) ;
 			partition .set_sector_usage( T, N ) ;
+			partition.fs_block_size = S;
 		}
 	}
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -122,10 +123,10 @@ void jfs::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -144,10 +145,10 @@ void jfs::read_uuid( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
