@@ -17,6 +17,8 @@
  
  
 #include "../include/linux_swap.h"
+#include "../include/BlockSpecial.h"
+#include "../include/Partition.h"
 
 #include <cerrno>
 
@@ -41,8 +43,7 @@ const Glib::ustring linux_swap::get_custom_text( CUSTOM_TEXT ttype, int index )
 
 FS linux_swap::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = GParted::FS_LINUX_SWAP ;
+	FS fs( FS_LINUX_SWAP );
 
 	fs .busy = FS::GPARTED ;
 	fs .read = FS::EXTERNAL ;
@@ -79,13 +80,13 @@ void linux_swap::set_used_sectors( Partition & partition )
 		std::ifstream input( "/proc/swaps" ) ;
 		if ( input )
 		{
-			Glib::ustring path = partition .get_path() ;
-			Glib::ustring::size_type path_len = path.length() ;
+			BlockSpecial bs_path = BlockSpecial( partition.get_path() );
 			while ( getline( input, line ) )
 			{
-				if ( line .substr( 0, path_len ) == path )
+				Glib::ustring filename = Utils::regexp_label( line, "^([[:graph:]]+)" );
+				if ( bs_path == BlockSpecial( filename ) )
 				{
-					sscanf( line.substr( path_len ).c_str(), " %*s %*d %Ld", &N );
+					sscanf( line.c_str(), "%*s %*s %*d %lld", &N );
 					break ;
 				}
 			}
@@ -93,7 +94,7 @@ void linux_swap::set_used_sectors( Partition & partition )
 		}
 		else
 		{
-			partition .messages .push_back( "open(\"/proc/swaps\", O_RDONLY): " + Glib::strerror( errno ) ) ;
+			partition.push_back_message( "open(\"/proc/swaps\", O_RDONLY): " + Glib::strerror( errno ) );
 		}
 		if ( N > -1 )
 		{
@@ -123,10 +124,10 @@ void linux_swap::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -145,10 +146,10 @@ void linux_swap::read_uuid( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
