@@ -17,6 +17,11 @@
  
  
 #include "../include/ntfs.h"
+#include "../include/OperationDetail.h"
+#include "../include/Partition.h"
+#include "../include/Utils.h"
+
+#include <glibmm/ustring.h>
 
 namespace GParted
 {
@@ -56,8 +61,7 @@ const Glib::ustring ntfs::get_custom_text( CUSTOM_TEXT ttype, int index ) const
 
 FS ntfs::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = GParted::FS_NTFS ;
+	FS fs( FS_NTFS );
 
 	fs .busy = FS::GPARTED ;
 
@@ -123,14 +127,14 @@ void ntfs::set_used_sectors( Partition & partition )
 		"ntfsresize --info --force --no-progress-bar " + partition .get_path(), output, error, true ) ;
 	if ( exit_status == 0 || exit_status == 1 )
 	{
-		index = output .find( "Current volume size:" ) ;
+		Glib::ustring::size_type index = output.find( "Current volume size:" );
 		if ( index >= output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "Current volume size: %Ld", &T ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "Current volume size: %lld", &T ) != 1 )
 			T = -1 ;
 
 		index = output .find( "resize at" ) ;
 		if ( index >= output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "resize at %Ld", &N ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "resize at %lld", &N ) != 1 )
 			N = -1 ;
 		//For an absolutely full NTFS, "ntfsresize --info" exits
 		//  with status 1 and reports this message instead
@@ -138,20 +142,27 @@ void ntfs::set_used_sectors( Partition & partition )
 		if ( index < output .length() )
 			N = T ;
 
+		index = output.find( "Cluster size" );
+		if ( index >= output.length() ||
+		     sscanf( output.substr( index ).c_str(), "Cluster size       : %lld", &S ) != 1 )
+			S = -1;
+
 		if ( T > -1 && N > -1 )
 		{
 			T = Utils::round( T / double(partition .sector_size) ) ;
 			N = Utils::round( N / double(partition .sector_size) ) ;
 			partition .set_sector_usage( T, T - N );
 		}
+		if ( S > -1 )
+			partition.fs_block_size = S;
 	}
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -164,10 +175,10 @@ void ntfs::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -223,9 +234,9 @@ bool ntfs::resize( const Partition & partition_new, OperationDetail & operationd
 		//real resize
 		operationdetail .add_child( OperationDetail( _("real resize") ) ) ;
 
-		sigc::connection c = signal_progress.connect( sigc::mem_fun( *this, &ntfs::resize_progress ) );
 		if ( ! execute_command( cmd + " " + partition_new.get_path(),
-		                        operationdetail.get_last_child(), EXEC_CHECK_STATUS ) )
+		                        operationdetail.get_last_child(), EXEC_CHECK_STATUS|EXEC_PROGRESS_STDOUT,
+		                        static_cast<StreamSlot>( sigc::mem_fun( *this, &ntfs::resize_progress ) ) ) )
 		{
 			operationdetail .get_last_child() .set_status( STATUS_SUCCES ) ;
 			return_value = true ;
@@ -234,7 +245,6 @@ bool ntfs::resize( const Partition & partition_new, OperationDetail & operationd
 		{
 			operationdetail .get_last_child() .set_status( STATUS_ERROR ) ;
 		}
-		c.disconnect();
 	}
 	else
 	{
@@ -250,7 +260,8 @@ bool ntfs::copy( const Partition & src_part,
 {
 	return ! execute_command( "ntfsclone -f --overwrite " + dest_part.get_path() + " " + src_part.get_path(),
 	                          operationdetail,
-	                          EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
+	                          EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE|EXEC_PROGRESS_STDOUT,
+		                  static_cast<StreamSlot>( sigc::mem_fun( *this, &ntfs::clone_progress ) ) );
 }
 
 bool ntfs::check_repair( const Partition & partition, OperationDetail & operationdetail )
@@ -262,17 +273,44 @@ bool ntfs::check_repair( const Partition & partition, OperationDetail & operatio
 
 void ntfs::resize_progress( OperationDetail *operationdetail )
 {
-	size_t p = output.find_last_of('\n');
-	if ( p == output.npos )
-		return;
-	Glib::ustring ss = output.substr( p );
-	// looks like "12.34 percent completed"
-	float frac;
-	if ( sscanf( ss.c_str(), "%f percent completed", &frac ) == 1 )
+	Glib::ustring line = Utils::last_line( output );
+	// Text progress on the LAST LINE looks like " 15.24 percent completed"
+	// NOTE:
+	// Specifying text to match following the last converted variable in *scanf() is
+	// ineffective because it reports the number of successfully converted variables
+	// and has no way to indicate following text didn't match.  Specifically:
+	//     sscanf( "%f percent completed", &percent )
+	// will return 1 after successfully converting percent variable as 4 given this
+	// line from the important information at the end of the ntfsresize output:
+	//     "  4)  set the bootable flag for the partit"
+	// Definitely not progress information.
+	float percent;
+	if ( line.find( "percent completed" ) != line.npos && sscanf( line.c_str(), "%f", &percent ) == 1 )
 	{
-		operationdetail->fraction = frac / 100;
-		operationdetail->signal_update( *operationdetail );
+		operationdetail->run_progressbar( percent, 100.0 );
+	}
+	// Or when finished, on any line, ...
+	else if ( output.find( "Successfully resized NTFS on device" ) != output.npos )
+	{
+		operationdetail->stop_progressbar();
 	}
 }
 
+void ntfs::clone_progress( OperationDetail *operationdetail )
+{
+	Glib::ustring line = Utils::last_line( output );
+	// Text progress on the LAST LINE looks like " 15.24 progress completed"
+	float percent;
+	if ( line.find( "percent completed" ) != line.npos && sscanf( line.c_str(), "%f", &percent ) == 1 )
+	{
+		operationdetail->run_progressbar( percent, 100.0 );
+	}
+	// Deliberately don't stop the progress bar when ntfsclone outputs "Syncing ..."
+	// at the end as that is considered a measured part of the copy operation.  The
+	// progress bar will wait at 100% (or just below) until the sync completes.  On
+	// spinning rust that is typically a few seconds and on SSDs it won't be noticed
+	// at all.  Instead it is left for execute_command(), which always stops the
+	// progress bar when the command finishes.
+}
+
 } //GParted
