@@ -17,14 +17,14 @@
  
  
 #include "../include/reiserfs.h"
+#include "../include/Partition.h"
 
 namespace GParted
 {
 
 FS reiserfs::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = GParted::FS_REISERFS ;
+	FS fs( FS_REISERFS );
 
 	fs .busy = FS::GPARTED ;
 
@@ -81,19 +81,19 @@ void reiserfs::set_used_sectors( Partition & partition )
 {
 	if ( ! Utils::execute_command( "debugreiserfs " + partition .get_path(), output, error, true ) )
 	{
-		index = output .find( "Count of blocks on the device:" ) ;
+		Glib::ustring::size_type index = output.find( "Count of blocks on the device:" );
 		if ( index >= output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "Count of blocks on the device: %Ld", &T ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "Count of blocks on the device: %lld", &T ) != 1 )
 			T = -1 ;
 
 		index = output .find( "Blocksize:" ) ;
 		if ( index >= output .length() || 
-		     sscanf( output .substr( index ) .c_str(), "Blocksize: %Ld", &S ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "Blocksize: %lld", &S ) != 1 )
 			S = -1 ;
 
 		index = output .find( ":", output .find( "Free blocks" ) ) +1 ;
 		if ( index >= output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "%Ld", &N ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "%lld", &N ) != 1 )
 			N = -1 ;
 
 		if ( T > -1 && N > -1 && S > -1 )
@@ -101,15 +101,16 @@ void reiserfs::set_used_sectors( Partition & partition )
 			T = Utils::round( T * ( S / double(partition .sector_size) ) ) ;
 			N = Utils::round( N * ( S / double(partition .sector_size) ) ) ;
 			partition .set_sector_usage( T, N ) ;
+			partition.fs_block_size = S;
 		}
 	}
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -122,10 +123,10 @@ void reiserfs::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 	
@@ -145,10 +146,10 @@ void reiserfs::read_uuid( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
