@@ -17,14 +17,19 @@
  
  
 #include "../include/xfs.h"
+#include "../include/OperationDetail.h"
+#include "../include/Partition.h"
+#include "../include/ProgressBar.h"
+#include "../include/Utils.h"
+
+#include <glibmm/ustring.h>
 
 namespace GParted
 {
 
 FS xfs::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = GParted::FS_XFS ;
+	FS fs( FS_XFS );
 
 	fs .busy = FS::GPARTED ;
 
@@ -90,19 +95,19 @@ void xfs::set_used_sectors( Partition & partition )
 			true ) )
 	{
 		//blocksize
-		if ( sscanf( output .c_str(), "blocksize = %Ld", &S ) != 1 )
+		if ( sscanf( output.c_str(), "blocksize = %lld", &S ) != 1 )
 			S = -1 ;
 
 		//filesystem blocks
-		index = output .find( "\ndblocks" ) ;
+		Glib::ustring::size_type index = output.find( "\ndblocks" );
 		if ( index > output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "\ndblocks = %Ld", &T ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "\ndblocks = %lld", &T ) != 1 )
 			T = -1 ;
 
 		//free blocks
 		index = output .find( "\nfdblocks" ) ;
 		if ( index > output .length() ||
-		     sscanf( output .substr( index ) .c_str(), "\nfdblocks = %Ld", &N ) != 1 )
+		     sscanf( output.substr( index ).c_str(), "\nfdblocks = %lld", &N ) != 1 )
 			N = -1 ;
 
 		if ( T > -1 && N > -1 && S > -1 )
@@ -110,16 +115,17 @@ void xfs::set_used_sectors( Partition & partition )
 			T = Utils::round( T * ( S / double(partition .sector_size) ) ) ;
 			N = Utils::round( N * ( S / double(partition .sector_size) ) ) ;
 			partition .set_sector_usage( T, N ) ;
+			partition.fs_block_size = S;
 		}
 
 	}
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -132,10 +138,10 @@ void xfs::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -158,10 +164,10 @@ void xfs::read_uuid( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -223,7 +229,7 @@ bool xfs::copy( const Partition & src_part,
 	if ( src_mount_point .empty() )
 		return false ;
 
-	Glib::ustring dest_mount_point = mk_temp_dir( "dest", operationdetail ) ;
+	dest_mount_point = mk_temp_dir( "dest", operationdetail );
 	if ( dest_mount_point .empty() )
 	{
 		rm_temp_dir( src_mount_point, operationdetail ) ;
@@ -233,6 +239,14 @@ bool xfs::copy( const Partition & src_part,
 	success &= ! execute_command( "mount -v -t xfs -o noatime,ro " + src_part.get_path() +
 	                              " " + src_mount_point, operationdetail, EXEC_CHECK_STATUS );
 
+	// Get source FS used bytes, needed in progress update calculation
+	Byte_Value fs_size;
+	Byte_Value fs_free;
+	if ( Utils::get_mounted_filesystem_usage( src_mount_point, fs_size, fs_free, error ) == 0 )
+		src_used = fs_size - fs_free;
+	else
+		src_used = -1LL;
+
 	if ( success )
 	{
 		success &= ! execute_command( "mount -v -t xfs " + dest_part.get_path() +
@@ -242,7 +256,9 @@ bool xfs::copy( const Partition & src_part,
 		{
 			success &= ! execute_command( "sh -c 'xfsdump -J - " + src_mount_point +
 			                              " | xfsrestore -J - " + dest_mount_point + "'",
-			                              operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
+			                              operationdetail,
+			                              EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE|EXEC_PROGRESS_TIMED,
+			                              static_cast<TimedSlot>( sigc::mem_fun( *this, &xfs::copy_progress ) ) );
 
 			success &= ! execute_command( "umount -v " + dest_part.get_path(), operationdetail,
 			                              EXEC_CHECK_STATUS );
@@ -264,4 +280,30 @@ bool xfs::check_repair( const Partition & partition, OperationDetail & operation
 	                          EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
 }
 
+//Private methods
+
+// Report progress of XFS copy.  Monitor destination FS used bytes and track against
+// recorded source FS used bytes.
+bool xfs::copy_progress( OperationDetail * operationdetail )
+{
+	if ( src_used <= 0LL )
+	{
+		operationdetail->stop_progressbar();
+		// Failed to get source FS used bytes.  Remove this timed callback early.
+		return false;
+	}
+	Byte_Value fs_size;
+	Byte_Value fs_free;
+	Byte_Value dst_used;
+	if ( Utils::get_mounted_filesystem_usage( dest_mount_point, fs_size, fs_free, error ) != 0 )
+	{
+		operationdetail->stop_progressbar();
+		// Failed to get destination FS used bytes.  Remove this timed callback early.
+		return false;
+	}
+	dst_used = fs_size - fs_free;
+	operationdetail->run_progressbar( (double)dst_used, (double)src_used, PROGRESSBAR_TEXT_COPY_BYTES );
+	return true;
+}
+
 } //GParted
